/** \file main.c
	\author Tomasz Ostrowski
	\date August 2007
	
	Main file of SHT11 sensor library test project.
	Compiler: avr-gcc (WinAVR 20060421)
	Based on Sensirion sample code app note, rev. 2.05
	MCU: ATmega8, 4MHz crystal, Vcc = 5V (important for SHT11 calculations)
	sensor: SHT11, DATA with external 10k pullup connected to PD4, SCK connected to PD3
	MCU <-> PC: UART
	HFUSE/LFUSE: 0xC9/0x9F
*/

#include "main.h"	//IMPORTANT: it contains F_CPU definition, so include it before delay.h
#include <stdlib.h>
#include <avr/io.h>
#include <avr/delay.h>
#include <compat/deprecated.h>

#include "usart.h"
#include "SHTxx.h"


/** \brief just beep
*/
void testbeep(void)	
	{
	sbi(PORTC,5);
	_delay_loop_2(0xFFFF);				
	cbi(PORTC,5);		
	_delay_loop_2(0xFFFF);				
	}


int main(void)
{
	value humid_val,temp_val;
	unsigned char checksum, error = 0;
	char buf[20];
	
	sbi(DDRC,5);			//set test buzzer pin as output
	testbeep();

    usart_init(USART_BAUDRATE(9600, F_CPU/1000000UL));	//macro from usart.h, br = baudrate, fosc = clock frequency in megahertzs
	usart_puts("STH11 test app started.\r\n");
	
	for(;;)
	{
		//testbeep();
		
		//according to app note SHTxx should not be active more than 10% of the time to reduse self-heating to < 0.1 deg				
		//also, no command should be sent to SHT for 11ms after power-up
		for(unsigned char i=0; i<100; i++)
			_delay_ms(10);	//WARNING, max param 262.14ms/F_CPU

		s_connectionreset();
		
		//WARNING! s_measure may take up to 2s (max timeout), so you'll probably
		//have to rewrite this function into non-blocking code if your application does sth time-critical
		error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP);
		error+=s_measure((unsigned char*) &humid_val.i,&checksum,HUMID);
		if(error != 0)
		{ 
		   s_connectionreset();            
		   usart_puts("MEAS ERROR\r\n");
		}
		else
		{
		   humid_val.f=(float)humid_val.i;
		   temp_val.f=(float)temp_val.i;
		   calc_sht11(&humid_val.f,&temp_val.f);
		   temp_val.i=(int32_t)(temp_val.f*10);
		   
		   usart_puts("Temp: ");
		   if(temp_val.i < 0)
				usart_putc('-');
			temp_val.i = abs(temp_val.i);
		   itoa(temp_val.i/10, buf, 10);
		   usart_puts(buf);
		   usart_putc('.');
		   itoa(temp_val.i%10, buf, 10);
		   usart_puts(buf);
		   usart_puts("\r\n");
		   
		   humid_val.i = (int32_t)(humid_val.f);
		   usart_puts("RH: ");
		   itoa(humid_val.i, buf, 10);
		   usart_puts(buf);
		   usart_putc('%');
		   usart_puts("\r\n\r\n");   
		}
	}
}